from cloud_governance.common.clouds.aws.price.price import AWSPrice
from cloud_governance.common.utils.configs import DEFAULT_ROUND_DIGITS
from cloud_governance.main.environment_variables import environment_variables


class ResourcesPricing:
    """
    This class calculates the AWS resources pricing
    """

    MONTHLY_HOURS = 730
    IP_HOURLY_COST = 0.005

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self._aws_pricing = AWSPrice()
        self.region = self.__environment_variables_dict.get('AWS_DEFAULT_REGION', 'us-east-1')

    def ec2_instance_type_cost(self, instance_type: str, hours: float):
        """
        This method returns the cost of ec2 instance types cost
        @return:
        """
        cost = float(self._aws_pricing.get_price(instance=instance_type, os='Linux',
                                                 region=self._aws_pricing.get_region_name(self.region)))
        return cost * hours

    def get_ebs_cost(self, volume_size: int, volume_type: str, hours: float):
        """
        This method returns the cost of ebs_volume
        @param hours:
        @param volume_size:
        @param volume_type:
        @return:
        """
        cost = float(self._aws_pricing.get_ebs_cost(volume_type=volume_type, region=self.region))
        return cost * volume_size * (hours / self.MONTHLY_HOURS)

    def get_const_prices(self, resource_type: str, hours: int):
        """
        This method gives the cost of const resources
        @param hours:
        @param resource_type:
        @return:
        """
        if resource_type == 'eip':
            return self.IP_HOURLY_COST * hours

    def get_eip_unit_price(self):
        """
        This method returns the ElasticIp Price
        :return:
        :rtype:
        """
        return self.IP_HOURLY_COST

    def get_nat_gateway_unit_price(self, region_name: str):
        """
        This method returns the unit price of NatGateway
        :param region_name:
        :type region_name:
        :return:
        :rtype:
        """
        service_code = 'AmazonEC2'
        filter_dict = [
            {"Field": "productFamily", "Value": "NAT Gateway", "Type": "TERM_MATCH"},
            {"Field": "regionCode", "Value": region_name, "Type": "TERM_MATCH"},
            {"Field": "groupDescription", "Value": "Hourly charge for NAT Gateways", "Type": "TERM_MATCH"},
        ]
        unit_price = self._aws_pricing.get_service_pricing(service_code, filter_dict)
        return round(unit_price, DEFAULT_ROUND_DIGITS)

    def get_ebs_unit_price(self, region_name: str, ebs_type: str):
        """
        This method returns the ebs_type unit price
        :param region_name:
        :type region_name:
        :param ebs_type:
        :type ebs_type:
        :return:
        :rtype:
        """
        service_code = 'AmazonEC2'
        filter_dict = [
            {"Field": "productFamily", "Value": "Storage", "Type": "TERM_MATCH"},
            {"Field": "regionCode", "Value": region_name, "Type": "TERM_MATCH"},
            {"Field": "volumeApiName", "Value": ebs_type, "Type": "TERM_MATCH"},
        ]
        unit_price = self._aws_pricing.get_service_pricing(service_code, filter_dict)
        return round(unit_price, DEFAULT_ROUND_DIGITS)

    def get_rds_price(self, region_name: str, instance_type: str):
        service_code = 'AmazonRDS'
        filter_dict = [
            {"Field": "productFamily", "Value": "Database Instance", "Type": "TERM_MATCH"},
            {"Field": "regionCode", "Value": region_name, "Type": "TERM_MATCH"},
            {"Field": "instanceType", "Value": instance_type, "Type": "TERM_MATCH"}
        ]
        unit_price = self._aws_pricing.get_service_pricing(service_code, filter_dict)
        return round(unit_price, DEFAULT_ROUND_DIGITS)

    def get_snapshot_unit_price(self, region_name: str):
        """
        This method returns the unit price of Ebs Snapshot
        :param region_name:
        :return:
        """
        service_code = 'AmazonEC2'
        filter_dict = [
            {"Field": "regionCode", "Value": region_name, "Type": "TERM_MATCH"},
            {"Field": "productFamily", "Value": "Storage Snapshot", "Type": "TERM_MATCH"},
            {"Field": "snapshotarchivefeetype", "Value": "SnapshotArchiveStorage", "Type": "TERM_MATCH"},
        ]
        unit_price = self._aws_pricing.get_service_pricing(service_code, filter_dict)
        return round(unit_price, DEFAULT_ROUND_DIGITS)

    def get_ec2_price(self, region_name: str, instance_type: str, operating_system: str = None):
        """
        This method returns the unit price of Ec2 Price
        :param operating_system:
        :param region_name:
        :param instance_type:
        :return:
        """
        if not operating_system:
            operating_system = 'Linux/UNIX'
        os_types = {'Linux/UNIX': 'Linux',
                    'Red Hat Enterprise Linux': 'RHEL',
                    'SUSE Linux': 'SUSE',
                    'Ubuntu Pro Linux': 'Ubuntu Pro',
                    'Windows': 'Windows',
                    'Red Hat Enterprise Linux with High Availability': 'Red Hat Enterprise Linux with HA'}
        operating_system_value = "NA"
        for os_type in os_types.keys():
            if os_type.lower() == operating_system.lower():
                operating_system_value = os_types[os_type]
        service_code = 'AmazonEC2'
        filter_dict = [
            {"Field": "regionCode", "Value": region_name, "Type": "TERM_MATCH"},
            {"Field": "tenancy", "Value": "shared", "Type": "TERM_MATCH"},
            {"Field": "operatingSystem", "Value": f"{operating_system_value}", "Type": "TERM_MATCH"},
            {"Field": "instanceType", "Value": f"{instance_type}", "Type": "TERM_MATCH"},
            {"Field": "capacitystatus", "Value": "Used", "Type": "TERM_MATCH"},
        ]
        unit_price = self._aws_pricing.get_service_pricing(service_code, filter_dict)
        return round(unit_price, DEFAULT_ROUND_DIGITS)



from datetime import datetime

from cloud_governance.cloud_resource_orchestration.common.cro_object import CroObject
from cloud_governance.cloud_resource_orchestration.utils.common_operations import string_equal_ignore_case
from cloud_governance.common.elasticsearch.elasticsearch_operations import ElasticSearchOperations
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.main.environment_variables import environment_variables


class RunCRO:
    """This class monitors cro activities"""

    PERSISTENT_RUN_DOC_ID = f'cro_run_persistence-{datetime.utcnow().date()}'
    PERSISTENT_RUN_INDEX = 'cloud_resource_orchestration_persistence_run'

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__es_operations = ElasticSearchOperations()
        self.__account = self.__environment_variables_dict.get('account', '').lower()
        self.__cloud_name = self.__environment_variables_dict.get('PUBLIC_CLOUD_NAME')
        self.__cro_object = CroObject(public_cloud_name=self.__cloud_name)
        self.__cost_over_usage = self.__cro_object.cost_over_usage()
        self.__cro_reports = self.__cro_object.collect_cro_reports()
        self.__monitor_tickets = self.__cro_object.monitor_tickets()

    @logger_time_stamp
    def save_current_timestamp(self):
        """
        This method saves the current timestamp
        Storing timestamp for not sending multiple alerts in a day, if we run any number of times
        :return:
        """
        if not self.__es_operations.verify_elastic_index_doc_id(index=self.PERSISTENT_RUN_INDEX,
                                                                doc_id=self.PERSISTENT_RUN_DOC_ID):
            self.__es_operations.upload_to_elasticsearch(index=self.PERSISTENT_RUN_INDEX, data={
                f'last_run_{self.__account.lower()}': datetime.utcnow()}, id=self.PERSISTENT_RUN_DOC_ID)
        else:
            self.__es_operations.update_elasticsearch_index(index=self.PERSISTENT_RUN_INDEX,
                                                            metadata={f'last_run_{self.__account.lower()}': datetime.utcnow()},
                                                            id=self.PERSISTENT_RUN_DOC_ID)

    @logger_time_stamp
    def __send_cro_alerts(self):
        """
        This method sends the cost_over_usage alert and Ticket status alerts
        :return:
        """
        es_data = self.__es_operations.get_es_data_by_id(index=self.PERSISTENT_RUN_INDEX, id=self.PERSISTENT_RUN_DOC_ID)
        first_run = True
        try:
            if es_data:
                source = es_data.get('_source')
                last_run_time = source.get(f'last_run_{self.__account.lower()}')
                if last_run_time:
                    last_updated_time = datetime.strptime(last_run_time, "%Y-%m-%dT%H:%M:%S.%f").date()
                    if last_updated_time == datetime.utcnow().date():
                        first_run = False
            self.__environment_variables_dict.update({'CRO_FIRST_RUN': first_run})
            if first_run:
                cost_over_usage_users = self.__cost_over_usage.run()
                logger.info(f'Cost Over Usage Users list: {cost_over_usage_users}')
                self.__monitor_tickets.run()
                self.__cro_reports.update_in_progress_ticket_cost()
        except Exception as err:
            logger.error(err)
        self.save_current_timestamp()

    def __run_cloud_resources(self):
        """
        This method runs the public cloud resources and upload results to es
        :return:
        :rtype:
        """
        active_regions = self.__cro_object.get_active_regions()
        logger.info(f"""***** Running CloudResourceOrchestration in all Active regions: {active_regions} *****""")
        for active_region in active_regions:
            cro_monitor = self.__cro_object.get_monitor_cro_resources_object(region_name=active_region)
            cro_tagging = self.__cro_object.get_tag_cro_resources_object(region_name=active_region)
            if string_equal_ignore_case(self.__cloud_name, 'aws'):
                self.__environment_variables_dict.update({'AWS_DEFAULT_REGION': active_region})
            logger.info(f"""Running CloudResourceOrchestration in region: {active_region}""")
            logger.info(f"""{active_region}: -> Running CRO Tagging""")
            tagging_response = cro_tagging.run()
            logger.info(f'Tagged instances : {tagging_response}')
            logger.info(f"""{active_region}: -> Running CRO Resource data Collection""")
            monitor_response = cro_monitor.run()
            if monitor_response:
                cro_reports = self.__cro_reports.run(monitor_response)
                logger.info(f'Cloud Orchestration Resources: {cro_reports}')

    @logger_time_stamp
    def __start_cro(self):
        """
        This method starts the cro process methods
        1. Send alert to cost over usage users
        2. Tag the new instances
        3. monitor and upload the new instances' data
        4. Monitor the Jira ticket progressing
        :return:
        """
        self.__send_cro_alerts()
        self.__run_cloud_resources()

    @logger_time_stamp
    def run(self):
        """
        This method starts the aws CRO operations
        :return:
        """
        self.__start_cro()



from datetime import datetime

from cloud_governance.policy.helpers.azure.azure_policy_operations import AzurePolicyOperations


class InstanceRun(AzurePolicyOperations):

    INSTANCE_TYPES_ES_INDEX = "cloud-governance-instance-types"
    RESOURCE_ACTION = "Stopped"

    def __init__(self):
        super().__init__()

    def _upload_instance_type_count_to_elastic_search(self):
        """
        This method uploads the instance type count to elasticsearch
        :return:
        :rtype:
        """
        instance_types = self._update_instance_type_count()
        account = self.account
        current_day = datetime.utcnow()
        es_instance_types_data = []
        for region, instance_types in instance_types.items():
            for instance_type, instance_type_count in instance_types.items():
                es_instance_types_data.append({
                    'instance_type': instance_type,
                    'instance_count': instance_type_count,
                    'timestamp': current_day,
                    'region': region,
                    'account': account,
                    'PublicCloud': self._cloud_name,
                    'index_id': f'{instance_type}-{self._cloud_name.lower()}-{account.lower()}-{region}-{str(current_day.date())}'
                })
        self._es_upload.es_upload_data(items=es_instance_types_data, es_index=self.INSTANCE_TYPES_ES_INDEX,
                                       set_index='index_id')

    def _update_instance_type_count(self):
        """
        This method returns the instance type count by region
        :return: { region : {instance_type: instance_count} }
        :rtype: dict
        """
        resources = self._get_all_instances()
        instance_types = {}
        for resource in resources:
            vm_type = resource.hardware_profile.vm_size
            region = resource.location
            instance_types.setdefault(region, {}).update(
                {vm_type: instance_types.get(region).get(vm_type, 0) + 1}
            )
        return instance_types

    def run_policy_operations(self):
        """
        This method returns the running vms in the AAzure cloud and stops based on the action
        :return:
        :rtype:
        """
        self._upload_instance_type_count_to_elastic_search()
        vms_list = self._get_all_instances()
        running_vms = []
        for vm in vms_list:
            status = self._get_instance_status(resource_id=vm.id, vm_name=vm.name)
            tags = vm.tags if vm.tags else {}
            cleanup_result = False
            if 'running' in status:
                running_days = self.calculate_days(vm.time_created)
                if self._shutdown_period:
                    cleanup_days = self.get_clean_up_days_count(tags=tags)
                    cleanup_result = self.verify_and_delete_resource(resource_id=vm.id, tags=tags,
                                                                     clean_up_days=cleanup_days)
                else:
                    cleanup_days = 0
                resource_data = self._get_es_schema(
                    resource_id=vm.name,
                    skip_policy=self.get_skip_policy_value(tags=tags),
                    user=self.get_tag_name_from_tags(tags=tags, tag_name='User'),
                    launch_time=vm.time_created,
                    resource_type=vm.hardware_profile.vm_size,
                    resource_state=status if not cleanup_result else 'Vm Stopped',
                    running_days=running_days, cleanup_days=cleanup_days,
                    dry_run=self._dry_run,
                    name=vm.name,
                    resource_action=self.RESOURCE_ACTION,
                    region=vm.location, cleanup_result=str(cleanup_result),
                    cloud_name=self._cloud_name
                )
                if self._shutdown_period and self._force_delete and self._dry_run == 'no':
                    resource_data.update({'ForceDeleted': str(self._force_delete)})
                running_vms.append(resource_data)
            else:
                cleanup_days = 0
            if self._shutdown_period:
                self.update_resource_day_count_tag(resource_id=vm.id, cleanup_days=cleanup_days, tags=tags)
        return running_vms



import boto3

from cloud_governance.common.logger.init_logger import logger


class ResourceTagAPIOperations:
    """
    This class performs the resourcegrouptagapi operations
    """

    PAGINATION_TOKEN = 'PaginationToken'

    def __init__(self, region_name: str):
        self.__client = boto3.client('resourcegroupstaggingapi', region_name=region_name)

    def get_resources(self, tag_name: str, tag_value: str = ''):
        """
        This method returns all the resources having the tag_name and tag_value
        :return:
        """
        resources_list = []
        filters = {'Key': tag_name, 'Values': []}
        if tag_value:
            filters['Values'].append(tag_value)
        try:
            response = self.__client.get_resources(TagFilters=[filters])
            resources_list.extend(response['ResourceTagMappingList'])
            while response.get(self.PAGINATION_TOKEN):
                response = self.__client.get_resources(TagFilters=[filters],
                                                       PaginationToken=response.get(self.PAGINATION_TOKEN))
                resources_list.extend(response['ResourceTagMappingList'])
        except Exception as err:
            logger.error(err)
            raise err
        return resources_list

    def tag_resources(self, resource_arn_list: list, update_tags_dict: dict):
        """
        This method updates/tags list to the given resource arn's
        :param resource_arn_list:
        :param update_tags_dict:  {key: value}
        :return:
        """
        try:
            self.__client.tag_resources(ResourceARNList=resource_arn_list, Tags=update_tags_dict)
        except Exception as err:
            logger.error(err)
            raise err




from cloud_governance.cloud_resource_orchestration.utils.common_operations import check_name_and_get_key_from_tags
from cloud_governance.common.clouds.aws.ec2.ec2_operations import EC2Operations
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.main.environment_variables import environment_variables


class MonitorCROInstances:
    """
    This class monitor cro instances
    """

    def __init__(self, region_name: str = ''):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__region_name = region_name if region_name else self.__environment_variables_dict.get('AWS_DEFAULT_REGION')
        self.__ec2_operations = EC2Operations(region=self.__region_name)
        self.__cro_resource_tag_name = self.__environment_variables_dict.get('CRO_RESOURCE_TAG_NAME')

    @logger_time_stamp
    def __monitor_instances(self):
        """
        This method monitor instances and returns the data
        :return:
        """
        monitored_ticket_ids = {}
        filters = {'Filters': [{'Name': 'tag-key', 'Values': ['Duration']}]}
        cluster_tickets = {}
        instances = self.__ec2_operations.get_ec2_instance_list(**filters)
        for resource in instances:
            tags = resource.get('Tags')
            ticket_id = self.__ec2_operations.get_tag_value_from_tags(tag_name=self.__cro_resource_tag_name, tags=tags)
            cluster_key, cluster_value = check_name_and_get_key_from_tags(tags=tags, tag_name='kubernetes.io/cluster/')
            hcp_key, hcp_name = check_name_and_get_key_from_tags(tags=tags, tag_name='api.openshift.com/name')
            if hcp_name:
                cluster_key = hcp_name
            rosa, rosa_value = check_name_and_get_key_from_tags(tags=tags, tag_name='red-hat-clustertype')
            if cluster_key:
                cluster_tickets.setdefault(ticket_id, {}).setdefault(cluster_key, {}).setdefault('instance_data', []) \
                    .append(f"{self.__ec2_operations.get_tag_value_from_tags(tag_name='Name', tags=tags)}: "
                            f"{resource.get('InstanceId')}: {resource.get('InstanceLifecycle', 'ondemand')}: "
                            f"{resource.get('InstanceType')}: {'rosa' if rosa else 'self'}")
                cluster_tickets.setdefault(ticket_id, {}).setdefault(cluster_key, {}). \
                    update({
                        'region_name': self.__region_name,
                        'ticket_id': ticket_id,
                        'user_cro': self.__ec2_operations.get_tag_value_from_tags(tag_name='UserCRO', tags=tags),
                        'user': self.__ec2_operations.get_tag_value_from_tags(tag_name='User', tags=tags),
                        'manager': self.__ec2_operations.get_tag_value_from_tags(tag_name='Manager', tags=tags),
                        'approved_manager': self.__ec2_operations.get_tag_value_from_tags(tag_name='ApprovedManager', tags=tags),
                        'owner': self.__ec2_operations.get_tag_value_from_tags(tag_name='Owner', tags=tags),
                        'project': self.__ec2_operations.get_tag_value_from_tags(tag_name='Project', tags=tags),
                        'email': self.__ec2_operations.get_tag_value_from_tags(tag_name='Email', tags=tags),
                        'duration': self.__ec2_operations.get_tag_value_from_tags(tag_name='Duration', tags=tags, cast_type='int'),
                        'estimated_cost': self.__ec2_operations.get_tag_value_from_tags(tag_name='EstimatedCost', tags=tags, cast_type='float')
                })
            else:
                monitored_ticket_ids.setdefault(ticket_id, []).append({
                    'instance_data': f"{self.__ec2_operations.get_tag_value_from_tags(tag_name='Name', tags=tags)}: "
                                     f"{resource.get('InstanceId')}: {resource.get('InstanceLifecycle', 'ondemand')}: "
                                     f"{resource.get('InstanceType')}",
                    'region_name': self.__region_name,
                    'ticket_id': ticket_id,
                    'user_cro': self.__ec2_operations.get_tag_value_from_tags(tag_name='UserCRO', tags=tags),
                    'user': self.__ec2_operations.get_tag_value_from_tags(tag_name='User', tags=tags),
                    'manager': self.__ec2_operations.get_tag_value_from_tags(tag_name='Manager', tags=tags),
                    'approved_manager': self.__ec2_operations.get_tag_value_from_tags(tag_name='ApprovedManager',
                                                                                      tags=tags),
                    'owner': self.__ec2_operations.get_tag_value_from_tags(tag_name='Owner', tags=tags),
                    'project': self.__ec2_operations.get_tag_value_from_tags(tag_name='Project', tags=tags),
                    'email': self.__ec2_operations.get_tag_value_from_tags(tag_name='Email', tags=tags),
                    'duration': self.__ec2_operations.get_tag_value_from_tags(tag_name='Duration', tags=tags,
                                                                              cast_type='int'),
                    'estimated_cost': self.__ec2_operations.get_tag_value_from_tags(tag_name='EstimatedCost', tags=tags,
                                                                                    cast_type='float')
                })
        for ticket_id, cluster_data in cluster_tickets.items():
            for cluster_id, cluster_values in cluster_data.items():
                cluster_values['cluster_name'] = cluster_id.split('/')[-1]
                monitored_ticket_ids.setdefault(ticket_id, []).append(cluster_values)
        return monitored_ticket_ids

    @logger_time_stamp
    def run(self):
        """
        This method run the monitoring methods
        :return:
        """
        return self.__monitor_instances()



import datetime
import operator
import tempfile
from operator import sub, add

import pandas as pd

from cloud_governance.common.clouds.aws.cost_explorer.cost_explorer_operations import CostExplorerOperations
from cloud_governance.common.clouds.aws.iam.iam_operations import IAMOperations
from cloud_governance.common.clouds.aws.sts.sts_oprations import STSOperations
from cloud_governance.common.elasticsearch.elastic_upload import ElasticUpload
from cloud_governance.common.google_drive.google_drive_operations import GoogleDriveOperations
from cloud_governance.main.environment_variables import environment_variables
from cloud_governance.common.google_drive.upload_to_gsheet import UploadToGsheet


class CostBillingReports:
    """
    This class is responsible for generation cost billing report for Budget, Actual, Forecast
    """

    GRANULARITY = 'MONTHLY'
    COST_METRIC = 'UNBLENDED_COST'
    MONTHS = 12
    END_DAY = 31
    FORECAST_DAYS = 360

    def __init__(self):
        try:
            self._environment_variables_dict = environment_variables.environment_variables_dict
            self.__cost_explorer_operations = CostExplorerOperations()
            self.elastic_upload = ElasticUpload()
            self.account_name, self.__cloud_name = IAMOperations().get_account_alias_cloud_name()
            self.__account_id = STSOperations().get_account_id()
            self.__gsheet_id = self._environment_variables_dict.get('SPREADSHEET_ID', '')
            self.gdrive_operations = GoogleDriveOperations()
            self.update_to_gsheet = UploadToGsheet()
            self.cost_center, self.__account_budget, self.__years, self.__owner = self.update_to_gsheet.get_cost_center_budget_details(account_id=self.__account_id)
        except:
            pass

    def __get_start_date(self, end_date: datetime, days: int, operation: operator) -> datetime:
        """
        This method returns start_date
        @param operation:
        @param end_date:
        @param days:
        @return:
        """
        return operation(end_date, datetime.timedelta(days=days))

    def get_date_ranges(self, days: int = 0):
        """
        This method returns the date ranges
        @param days:
        @return:
        """
        end_date = datetime.datetime.now()+datetime.timedelta(1)
        if days == 0:
            start_date = self.__get_start_date(end_date=end_date, days=self.END_DAY, operation=sub).replace(day=1).strftime('%Y-%m-%d')
        else:
            start_date = end_date.strftime('%Y-%m-%d')
            end_date = self.__get_start_date(end_date=end_date, days=days, operation=add)
        end_date = end_date.strftime('%Y-%m-%d')
        return start_date, end_date

    def filter_cost_usage_data(self, cost_usage_data: list, cost_metric: str):
        """
        This method filter and removes the unwanted data
        @param cost_metric:
        @param cost_usage_data:
        @return:
        """
        cost_data = {}
        for cost_usage in cost_usage_data:
            start_year = str(cost_usage.get('TimePeriod').get('Start')).split('-')[0]
            data = {}
            data['Actual'] = round(float(cost_usage.get('Total').get(cost_metric).get('Amount')), 3)
            data['Account'] = self.account_name
            data['start_date'] = str(cost_usage.get('TimePeriod').get('Start'))
            data['index_id'] = f"""{data['start_date']}-{data['Account'].lower()}"""
            data['timestamp'] = datetime.datetime.strptime(data['start_date'], '%Y-%m-%d')
            data['Month'] = datetime.datetime.strftime(data['timestamp'], '%Y %b')
            if start_year in self.__years:
                data['Budget'] = round(self.__account_budget/self.MONTHS, 3)
                data['AllocatedBudget'] = self.__account_budget
            else:
                data['Budget'] = 0
                data['AllocatedBudget'] = 0
            data['CostCenter'] = self.cost_center
            data['CloudName'] = self.__cloud_name
            data['Forecast'] = 0
            data['AccountId'] = self.__account_id
            data['filter_date'] = data['start_date']+data['Month'].strip()
            cost_data[data['start_date']] = data
        return cost_data

    def append_forecasting_data(self, cost_usage_data: dict, cost_forecast_data: list):
        """
        This method appends the fore
        @param cost_forecast_data:
        @param cost_usage_data:
        @return:
        """
        for cost_forecast in cost_forecast_data:
            start_date = str((cost_forecast.get('TimePeriod').get('Start')))
            cost = round(float(cost_forecast.get('MeanValue')), 3)
            start_year = str(cost_forecast.get('TimePeriod').get('Start')).split('-')[0]
            if start_date in cost_usage_data:
                cost_usage_data[start_date]['Forecast'] = cost
            else:
                data = {}
                data['AccountId'] = self.__account_id
                data['Actual'] = 0
                data['Forecast'] = cost
                data['Account'] = self.account_name
                data['start_date'] = str((cost_forecast.get('TimePeriod').get('Start')))
                data['index_id'] = f"""{data['start_date']}-{data['Account'].lower()}"""
                data['timestamp'] = datetime.datetime.strptime(data['start_date'], '%Y-%m-%d')
                data['Month'] = datetime.datetime.strftime(data['timestamp'], '%Y %b')
                if start_year in self.__years:
                    data['Budget'] = round(self.__account_budget / self.MONTHS, 3)
                    data['AllocatedBudget'] = self.__account_budget
                else:
                    data['Budget'] = 0
                    data['AllocatedBudget'] = 0
                data['CostCenter'] = self.cost_center
                data['CloudName'] = self.__cloud_name
                data['filter_date'] = data['start_date']+data['Month']
                cost_usage_data.setdefault(start_date, data)
        return cost_usage_data

    def get_cost_usage_of_month(self):
        """
        This method get cost usage of this month
        @rtype: object
        """
        start_date, end_date = self.get_date_ranges()
        cost_data = self.__cost_explorer_operations.get_cost_and_usage_from_aws(start_date=start_date, end_date=end_date, granularity=self.GRANULARITY)
        start_date, end_date = self.get_date_ranges(days=self.FORECAST_DAYS)
        cost_forecast_data = self.__cost_explorer_operations.get_cost_forecast(start_date=start_date, end_date=end_date, granularity=self.GRANULARITY, cost_metric=self.COST_METRIC)
        cost_filtered_data = self.filter_cost_usage_data(cost_usage_data=cost_data['ResultsByTime'], cost_metric=self.COST_METRIC.title().replace('_', ''))
        self.append_forecasting_data(cost_usage_data=cost_filtered_data, cost_forecast_data=cost_forecast_data['ForecastResultsByTime'])
        self.elastic_upload.es_upload_data(items=list(cost_filtered_data.values()), set_index='index_id')
        upload_data = {
            'cloud_alias_name': self.account_name,
            'cloud_name': 'AWS',
            'cloud_data': list(cost_filtered_data.values())
        }
        self.update_to_gsheet.update_data(cloud_data=upload_data)
        return cost_filtered_data

    def run(self):
        """
        This method run the methods
        @return:
        """
        self.get_cost_usage_of_month()



from cloud_governance.common.clouds.aws.utils.common_methods import get_boto3_client
from cloud_governance.common.clouds.aws.utils.utils import Utils
from cloud_governance.common.logger.init_logger import logger


class RDSOperations:
    """
    This class performs the RDS operations
    """

    def __init__(self, region_name: str):
        self._db_client = get_boto3_client('rds', region_name=region_name)

    def describe_db_instances(self, **kwargs):
        """
        This method returns the rds databases
        :return:
        :rtype:
        """
        rds_instances = []
        try:
            rds_instances = Utils.iter_client_function(func_name=self._db_client.describe_db_instances,
                                                       output_tag='DBInstances',
                                                       iter_tag_name='Marker', **kwargs)
        except Exception as err:
            logger.error(f"Can't describe the rds instances: {err}")
        return rds_instances

    def add_tags_to_resource(self, resource_arn: str, tags: list):
        """
        This method add/ update the tags to the database
        :param resource_arn:
        :type resource_arn:
        :param tags:
        :type tags:
        :return:
        :rtype:
        """
        try:
            self._db_client.add_tags_to_resource(ResourceName=resource_arn, Tags=tags)
            logger.info(f"Tags are updated to the resource: {resource_arn}")
        except Exception as err:
            logger.error(f"Something went wrong in add/ update tags: {err}")



from cloud_governance.common.utils.configs import INSTANCE_IDLE_DAYS
from cloud_governance.common.utils.utils import Utils
from cloud_governance.policy.helpers.aws.aws_policy_operations import AWSPolicyOperations


class InstanceIdle(AWSPolicyOperations):
    """
    This class stop the idle ec2 instances more than 4 days if the matches the required metrics
    CpuUtilization < 2 percent
    NetworkIN < 5k bytes
    NetworkOut < 5k bytes
    We trigger email if the ec2 instance is idle 2 days and stop the instance if it is idle 4 days
    """
    RESOURCE_ACTION = 'Stop'

    def __init__(self):
        super().__init__()

    def run_policy_operations(self):
        """
        This method returns the running instances
        :return:
        :rtype:
        """
        instances = self._get_all_instances()
        idle_instances = []
        for instance in instances:
            instance_id = instance.get('InstanceId')
            status = instance.get('State', {}).get('Name')
            tags = instance.get('Tags', [])
            cluster_tag = self._get_cluster_tag(tags=tags)
            cleanup_result = False
            running_days = self.calculate_days(instance.get('LaunchTime'))
            if Utils.contains_ignore_case(string=status, str1='running') and \
                    not cluster_tag and \
                    Utils.greater_than(val1=running_days, val2=INSTANCE_IDLE_DAYS) and \
                    self.get_skip_policy_value(tags=tags) not in ('NOTDELETE', 'SKIP') and \
                    self.verify_instance_idle(resource_id=instance_id):
                cleanup_days = self.get_clean_up_days_count(tags=tags)
                unit_price = self._resource_pricing.get_ec2_price(region_name=self._region,
                                                                  instance_type=instance.get('InstanceType'),
                                                                  operating_system=instance.get('PlatformDetails'))
                cleanup_result = self.verify_and_delete_resource(resource_id=instance_id, tags=tags,
                                                                 clean_up_days=cleanup_days)
                resource_data = self._get_es_schema(
                    resource_id=instance_id,
                    skip_policy=self.get_skip_policy_value(tags=tags),
                    user=self.get_tag_name_from_tags(tags=tags, tag_name='User'),
                    launch_time=instance['LaunchTime'].strftime("%Y-%m-%dT%H:%M:%S+00:00"),
                    resource_type=instance.get('InstanceType'),
                    resource_state=status if not cleanup_result else self.RESOURCE_ACTION,
                    running_days=running_days, cleanup_days=cleanup_days,
                    dry_run=self._dry_run,
                    name=self.get_tag_name_from_tags(tags=tags, tag_name='Name'),
                    resource_action=self.RESOURCE_ACTION,
                    region=self._region, cleanup_result=str(cleanup_result),
                    cloud_name=self._cloud_name,
                    unit_price=unit_price
                )
                if self._force_delete and self._dry_run == 'no':
                    resource_data.update({'ForceDeleted': str(self._force_delete)})
                idle_instances.append(resource_data)
            else:
                cleanup_days = 0
            if not cleanup_result:
                self.update_resource_day_count_tag(resource_id=instance_id, cleanup_days=cleanup_days, tags=tags)
        return idle_instances
